-- **************************************************************************
-- * TitanPals.lua
-- *
-- * By: Nagdand
-- *       (Nagdand@Laughing Skull)
-- **************************************************************************

-- ******************************** Constants *******************************
--TITAN_PALS_ID = "Pals";
-- ******************************** Variables *******************************
local L = LibStub("AceLocale-3.0"):GetLocale("Titan", true)
local LB = LibStub("AceLocale-3.0"):GetLocale("TitanPals", true)

-- ******************************** Functions *******************************

-- **************************************************************************
-- NAME : TitanPanelPalsButton_OnLoad()
-- DESC : Registers the plugin upon it loading
-- **************************************************************************
function TitanPanelPalsButton_OnLoad(self)
     self.registry = {
          id = LB["TITAN_PALS_CORE_ID"],
          builtIn = false,
          version = TITAN_VERSION,
          menuText = LB["TITAN_PALS_MENU_TEXT"],
          buttonTextFunction = "TitanPanelPalsButton_GetButtonText";
	  tooltipTitle = LB["TITAN_PALS_TOOLTIP"],
          tooltipCustomFunction = TitanPanelPalsButton_SetTooltip;
          icon = LB["TITAN_PALS_CORE_ARTWORK"],
          iconWidth = 16,
          category = "Information",
          savedVariables = {
	       ShowIcon = 1,
	       ShowLabelText = 1,
          }
     };
     self:RegisterEvent("PLAYER_ENTERING_WORLD");
     self:RegisterEvent("FRIENDLIST_UPDATE");
     self:RegisterEvent("IGNORELIST_UPDATE");
     self:RegisterEvent("VARIABLES_LOADED");
     self:RegisterEvent("PLAYER_REGEN_DISABLED");
     self:RegisterEvent("PLAYER_REGEN_ENABLED")
     TitanPanelPals_Init();
end


-- **************************************************************************
-- NAME : TitanPanelPals_Init()
-- DESC : For first time addon use
-- **************************************************************************
function TitanPanelPals_Init()
     working = {};
     if ( not TitanPals or TitanPals == nil or not TitanPals["Init"] ) then
          TitanPals = {};
  	  TitanPals["Init"] = true; --Init
	  TitanPals["Debug"] = false; --Debug
	  TitanPals["DebugEvent"] = false; --DebugEvent
	  TitanPals["DebugLogEvent"] = false; --DebugLogEvent
          TitanPals["ShowOffline"] = false; --ShowOffline
	  TitanPals["ShowIgnored"] = false; --ShowIgnored
	  TitanPals["ShowClass"] = true; --ShowClass
	  TitanPals["ShowNote"] = false; --ShowNote
	  TitanPals["ShowMem"] = true; --ShowMem
	  TitanPals["NoAlts"] = true; --NoAlts
	  TitanPals["UpDateDB"] = true; --UpDateDB
	  TitanPals["skip"] = 0; --skip
	  TitanPals["cIndex"] = 0; --cIndex
	  TitanPals["palsDB"] = ""; --palsDB
	  TitanPals["dLine"] = ""; --dLine
	  TitanPals["Lists"] = {}; --Lists
	  TitanPals["Temp"] = {}; --Temp
	  TitanPals["Alts"] = {};
     end
end

-- **************************************************************************
-- NAME : TitanPanelPalsButton_OnEvent()
-- DESC : Event Handler
-- **************************************************************************
function TitanPanelPalsButton_OnEvent(self, event, ...)
     if ( TitanPals["DebugEvent"] ) then TitanPanelPals_DebugUtils("bug", "OnEvent", "Event = "..event); end
     local Fac = UnitFactionGroup("player");
     local Rea = GetCVar("realmName");
     local Pla = GetUnitName("player");
     if ( TitanPals["cIndex"] ) then skip = TitanPals["cIndex"]; end
     TitanPanelButton_UpdateButton(LB["TITAN_PALS_CORE_ID"]);	
     TitanPanelButton_UpdateTooltip();
     if ( event == "VARIABLES_LOADED" ) then
          if ( not TitanPals["Player"] or TitanPals["Player"] ~= Pla ) then TitanPals["Player"] = Pla; end
	  if ( not TitanPals["faction"] or TitanPals["Faction"] ~= Fac ) then TitanPals["Faction"] = Fac; end
	  if ( not TitanPals["Realm"] or TitanPals["Realm"]~= Rea ) then TitanPals["Realm"] = Rea; end
	  if ( not TitanPals["palsDB"] or TitanPals["palsDB"] ~= Fac..":"..Rea ) then TitanPals["palsDB"] = Fac..":"..Rea; end
          TitanPanelPals_CoreUtils("GetcIndex", Fac, Rea, Pla);
          TitanPanelPals_CoreUtils("addAlt", Fac, Rea, Pla);
	  TitanPanelPals_DebugUtils("info", "load", "Titan Panel [|cffeda55fPals|r] |cff00aa00"..LB["TITAN_PALS_CORE_VERSION"].."|r by Nagand Loaded...")
	  TitanPanelPals_CoreUtils("mem");
     elseif ( event == "PLAYER_REGEN_DISABLED" ) then TitanPals["UpDateDB"] = false;
     elseif ( event == "PLAYER_REGEN_ENABLED" ) then TitanPals["UpDateDB"] = true;
     elseif ( event == "FRIENDLIST_UPDATE" ) then
	  if ( skip == 0 ) then
	       if ( TitanPals["UpDateDB"] == true ) then
	            TitanPanelPals_CoreUtils("UpDate", Fac, Rea, Pla);
	       end
	  end
	  if ( TitanPals["skip"] > 0 ) then TitanPals["skip"] = TitanPals["skip"] - 1; TitanPals["cIndex"] = TitanPals["cIndex"] - 1; end
     end
end

-- **************************************************************************
-- NAME : TitanPanelPalsButton_GetButtonText()
-- DESC : Build Button Text
-- **************************************************************************
function TitanPanelPalsButton_GetButtonText()
	local NumIgnore = GetNumIgnores();
	local CurCounts = TitanPanelPals_GetButtonCounts();
	local NumPals, NumPalsOnline = string.match( CurCounts, '(.*):(.*)' )
	-- create string for Titan bar display
	local buttonRichText = format(LB["TITAN_PALS_BUTTON_TEXT"], TitanPanelPals_ColorUtils("GetCText", "green", NumPalsOnline), TitanPanelPals_ColorUtils("GetCText", "hlight", NumPals), TitanPanelPals_ColorUtils("GetCText", "red", NumIgnore));
	return LB["TITAN_PALS_BUTTON_LABEL"], buttonRichText;
end

-- **************************************************************************
-- NAME : TitanPanelPalsButton_OnClick()
-- DESC : Toggles Friends Dialog
-- **************************************************************************
function TitanPanelPalsButton_OnClick(self, button)
	if ( button == "LeftButton" ) then
		ToggleFriendsFrame(1);
	else
		TitanPanelButton_OnClick(self, button);
	end
end

-- **************************************************************************
-- NAME : TitanPanelPals_GetButtonCounts()
-- DESC : Counts Friends, Ignored and Online for Button Text
-- **************************************************************************

function TitanPanelPals_GetButtonCounts()
	local np = GetNumFriends();
	local npo = 0;
	local pn, pl, pc, pa, pco, ps, pno, fI
	-- get a count of the number of online friends
	for fI=1, np do
		pn, pl, pc, pa, pco, ps, pno = GetFriendInfo(fI);
		if ( pco ) then npo = npo + 1; end
	end
	return np..":"..npo;
end


-- **************************************************************************
-- NAME : TitanPanelPals_SyncAlts()
-- DESC : This adds your alts to your friends list for things like mailing
-- **************************************************************************
function TitanPanelPals_SyncAlts()
     -- Alts to Pals List Syncing
     local NumPals = GetNumFriends();
     local AltsIndex = TitanPals["Alts"][TitanPals["palsDB"]];
     local Pals_name, Pals_level, Pals_class, Pals_area, Pals_connected, Pals_status, Pals_note
     for i=1, table.getn(AltsIndex) do
          TitanPanelPals_DebugUtils("bug", "SyncAlts", "i = "..i);
          local totalPals = 0;
          local friendsIndex
          for friendsIndex=1, NumPals do
               Pals_name, Pals_level, Pals_class, Pals_area, Pals_connected, Pals_status, Pals_note = GetFriendInfo(friendsIndex);
               if ( AltsIndex[i] == Pals_name ) then totalPals = 1; end
          end
          if ( totalPals == 0 and AltsIndex[i] ~= TitanPals["Player"] ) then
               TitanPanelPals_DebugUtils("bug", "SyncAlts", "Adding "..AltsIndex[i].." to firends list");
               AddFriend(AltsIndex[i]); 
	       TitanPals["cIndex"] = 1;
          end
     end
     TitanPanelPals_DebugUtils("info", "Sybc Alts", "Sync Finished.")
end

-- **************************************************************************
-- NAME : TitanPanelPals_SyncAlts()
-- DESC : This syncs your alts friend to yours
-- **************************************************************************

function TitanPanelPals_SyncAltsFriends()
     local NumPals = GetNumFriends();
     local iTable = TitanPals["Lists"][TitanPals["Faction"]][TitanPals["Realm"]]
     local Pals_name, Pals_level, Pals_class, Pals_area, Pals_connected, Pals_status, Pals_note
     for i=1, table.getn(iTable) do
          if ( iTable[i]["Name"] ~= TitanPals["Player"] ) then
               TitanPanelPals_DebugUtils("info", "Sync", "Syncing Alts Friends from : "..iTable[i]["Name"])
               local jTable = iTable[i];
	       for j=1, table.getn(jTable) do
	            local n, l, c, a = TitanPanelPals_DebugUtils("sArgs", jTable[j]);
                    local totalPals = 0;
                    local friendsIndex
                    for friendsIndex=1, NumPals do
                         Pals_name, Pals_level, Pals_class, Pals_area, Pals_connected, Pals_status, Pals_note = GetFriendInfo(friendsIndex);
                         if ( n == Pals_name ) then totalPals = 1; end
                    end
	            if ( totalPals == 0 and n ~= TitanPals["Player"] ) then
                         TitanPanelPals_DebugUtils("info", "SyncAltsFriends", "Adding "..n.." to friends list");
		         AddFriend(n);
	            end
	       end
	  end
     end
     TitanPanelPals_DebugUtils("info", "Sybc Alts", "Sync Finished.")
end